#!/usr/local/bin/clisp -C

;; Run this with LD_PRELOAD=/packages/freebsd/lib/libiconv.so !

(defun write-8-bit-table (name)
  (let* ((charset (make-encoding :charset name))
         (tmpfile "BIN256"))
    (unwind-protect
      (progn
        (with-open-file (f tmpfile :direction :output
                                   :element-type '(unsigned-byte 8))
          (dotimes (i 256) (write-byte i f))
        )
        (with-open-file (g (concatenate 'string name ".TXT") :direction :output)
          (format g "#~8TName:             ~A to Unicode~%" name)
          (format g "#~8TUnicode version:  3.0~%")
          (format g "#~8TTable version:    1.0~%")
          (format g "#~8TTable format:     Format A~%")
          (format g "#~8TDate:             ~1{~5@*~D-~4@*~2,'0D-~3@*~2,'0D~:}~%"
                    (multiple-value-list (get-decoded-time)))
          (format g "#       Generated by CLISP~%")
          (format g "#~%")
          (with-open-file (f tmpfile :direction :input :external-format charset)
            (dotimes (in 256)
              (let ((out #\uFFFD))
                (file-position f in)
                (ignore-errors (setq out (read-char f)))
                ; CLISP's line-terminator conversion is too intelligent.
                (when (and (eql in 13) (eql out #\u000A)) (setq out #\u000D))
                (unless (eql out #\uFFFD)
                  (format g "0x~2,'0X~C0x~4,'0X~C#~C~A~%"
                            in #\Tab (char-code out) #\Tab
                            #\Tab (sys::unicode-attributes out)
          ) ) ) ) )
          (format t "Created ~A~%" (pathname g))
        )
      )
      (delete-file tmpfile)
) ) )

(mapc #'write-8-bit-table '(
  "ASCII"
  "ISO646-GB"
  "ISO646-CA"
  "ISO646-CA2"
  "ISO646-DE"
  "ISO646-DK"
  "ISO646-ES"
  "ISO646-ES2"
;  "ISO646-CN"
  "ISO646-IT"
  "ISO646-JP"
  "ISO646-JP-OCR-A"
  "ISO646-JP-OCR-B"
  "ISO646-YU"
  "ISO646-KR"
  "ISO646-HU"
  "ISO646-CU"
  "ISO646-FR"
  "ISO646-FR1"
  "ISO646-NO"
  "ISO646-NO2"
  "ISO646-PT"
  "ISO646-PT2"
  "ISO646-SE"
  "ISO646-SE2"
  "ISO-8859-1"
  "ISO-8859-2"
  "ISO-8859-3"
  "ISO-8859-4"
  "ISO-8859-5"
  "ISO-8859-6"
  "ISO-8859-7"
  "ISO-8859-8"
  "ISO-8859-9"
  "ISO-8859-10"
  "ISO-8859-13"
  "ISO-8859-14"
  "ISO-8859-15"
  "T.61-7BIT"
  "T.61-8BIT"
;  "ISO_6937"
;  "ISO_6937-2"
;  "KOI-8"
  "KOI8-R"
  "KOI8-RU"
  "KOI8-U"
  "LATIN-GREEK"
  "LATIN-GREEK-1"
  "HP-ROMAN8"
  "EBCDIC-AT-DE"
  "EBCDIC-AT-DE-A"
  "EBCDIC-CA-FR"
  "EBCDIC-DK-NO"
  "EBCDIC-DK-NO-A"
  "EBCDIC-ES"
  "EBCDIC-ES-A"
  "EBCDIC-ES-S"
  "EBCDIC-FI-SE"
  "EBCDIC-FI-SE-A"
  "EBCDIC-FR"
;  "EBCDIC-IS-FRISS"
  "EBCDIC-IT"
  "EBCDIC-PT"
  "EBCDIC-UK"
  "EBCDIC-US"
  "IBM037"
  "IBM038"
;  "IBM256"
  "IBM273"
  "IBM274"
  "IBM275"
  "IBM277"
  "IBM278"
  "IBM280"
  "IBM281"
  "IBM284"
  "IBM285"
  "IBM290"
  "IBM297"
  "IBM420"
  "IBM423"
  "IBM424"
  "IBM437"
  "IBM500"
  "IBM737"
  "IBM775"
  "IBM850"
  "IBM851"
  "IBM852"
  "IBM855"
  "IBM857"
  "IBM860"
  "IBM861"
  "IBM862"
  "IBM863"
  "IBM864"
  "IBM865"
  "IBM866"
  "IBM868"
  "IBM869"
  "IBM870"
  "IBM871"
  "IBM874"
  "IBM880"
  "IBM891"
  "IBM903"
  "IBM904"
  "IBM905"
  "IBM918"
;  "IBM1004"
  "IBM1026"
;  "IBM1047"
  "CP856"
  "CP875"
  "CP1006"
  "CP1250"
  "CP1251"
  "CP1252"
  "CP1253"
  "CP1254"
  "CP1255"
  "CP1256"
  "CP1257"
  "CP1258"
  "MACINTOSH"
  "MAC-CE"
  "MAC-CROATIAN"
  "MAC-CYRILLIC"
  "MAC-DINGBATS"
  "MAC-GREEK"
  "MAC-ICELAND"
  "MAC-ROMAN"
  "MAC-ROMANIA"
  "MAC-THAI"
  "MAC-TURKISH"
  "MAC-UKRAINE"
  "IEC_P27-1"
;  "BALTIC"
  "ASMO_449"
  "ISO-IR-99"
  "ISO-IR-139"
;  "CWI"
  "DEC-MCS"
  "ECMA-CYRILLIC"
  "ISO-IR-153"
  "GREEK-CCITT"
  "GREEK7"
  "GREEK7-OLD"
  "INIS"
  "INIS-8"
  "INIS-CYRILLIC"
;  "ISO_2033"
  "ISO_2033-1983"
  "ISO_5427"
;  "ISO_5427-EXT"
;  "ISO_5428"
  "ISO_10367-BOX"
;  "MAC-IS"
;  "MAC-UK"
  "NATS-DANO"
  "NATS-DANO-ADD"
  "NATS-SEFI"
  "NATS-SEFI-ADD"
;  ; "SAMI-WS2" ??
;  "ISO-IR-197"
;  "TIS-620"

  "ADOBE-STDENC"
  "ADOBE-SYMBOL"
  "ADOBE-ZDINGBATS"
  "CP10000_MACROMAN"
  "CP10006_MACGREEK"
  "CP10007_MACCYRILLIC"
  "CP10029_MACLATIN2"
  "CP10079_MACICELANDIC"
  "CP10081_MACTURKISH"
  "ISO-IR-2"; "ISO_646.irv:1983", "irv",
  "ISO-IR-13"; "JIS_C6220-1969-jp", "JIS_C6220-1969", "katakana", "x0201-7",
  "ISO-IR-47"; "BS_viewdata",
  "ISO-IR-54"; "ISO_5427:1981",
  "ISO-IR-55"; "ISO_5428:1980",
  "ISO-IR-57"; "GB_1988-80",
  "ISO-IR-70"; "videotex-suppl",
  "ISO-IR-90";
  "ISO-IR-93"; "JIS_C6229-1984-b-add", "jp-ocr-b-add",
  "ISO-IR-94"; "JIS_C6229-1984-hand", "jp-ocr-hand",
  "ISO-IR-95"; "JIS_C6229-1984-hand-add", "jp-ocr-hand-add",
  "ISO-IR-96"; "JIS_C6229-1984-kana",
  "ISO-IR-123"; "CSA_Z243.4-1985-gr",
  "ISO-IR-128"; "T.101-G2",
  "ISO-IR-142"; "ISO_6937-2-add",
  "ISO-IR-146"; "JUS_I.B1.003-serb", "serbian",
  "ISO-IR-147"; "JUS_I.B1.003-mac", "macedonian",
  "ISO-IR-152"; "ISO_6937-2-25",
  "ISO-IR-154"; "ISO_8859-supp", "latin1-2-5",
  "ISO-IR-158"; "latin-lap", "lap",

  "JIS_X0201"
  ;"NEXTSTEP"

  ; also missing: UNICODEBIG and UNICODELITTLE aliases, and ucs4-internal

))
