/**
 * Create a table from CHARSET to Unicode. Based on the java.nio package.
 * This is a good test for the ByteToChar converter.
 * Note: This program doesn't currently work well: CharsetDecoder.decode
 * appears to be buggy for many encodings.
 *
 * @author Bruno Haible
 */
import java.io.*;
import java.nio.*;
import java.nio.charset.*;
public class table_from_nio {
  static String toHexString1 (int i) {
    return new String(new char[] { "0123456789ABCDEF".charAt(i) });
  }
  static String toHexString2 (int i) {
    return  toHexString1((i>>4)&0x0f)
           +toHexString1(i&0x0f);
  }
  static String toHexString4 (int i) {
    return  toHexString1((i>>12)&0x0f)
           +toHexString1((i>>8)&0x0f)
           +toHexString1((i>>4)&0x0f)
           +toHexString1(i&0x0f);
  }
  static void printOutput(CharBuffer out) {
    System.out.print("0x");
    for (int j = 0; j < out.length(); j++) {
      if (j > 0)
        System.out.print(" 0x");
      if (j+1 < out.length()
          && out.charAt(j) >= 0xd800 && out.charAt(j) < 0xdc00
          && out.charAt(j+1) >= 0xdc00 && out.charAt(j+1) < 0xe000) {
        int c = 0x10000 + ((out.charAt(j) - 0xd800) << 10) + (out.charAt(j+1) - 0xdc00);
        System.out.print(Integer.toHexString(c).toUpperCase());
        j++;
      } else
        System.out.print(toHexString4(out.charAt(j)));
    }
  }
  public static void main (String[] args) {
    if (args.length != 2) {
      System.err.println("Usage: java table_from_nio charset depth");
      System.exit(1);
    }
    String charset = args[0];
    int depth = Integer.parseInt(args[1]);
    CharsetDecoder converter = null;
    try {
      converter = Charset.forName(charset).newDecoder();
    } catch (IllegalArgumentException e) {
      System.err.println("no converter for "+charset);
      System.exit(1);
    }
    converter = converter.onMalformedInput(CodingErrorAction.REPORT);
    converter = converter.onUnmappableCharacter(CodingErrorAction.REPORT);
    for (int i0 = 0; i0 < 0x100; i0++) {
      try {
        byte[] in = new byte[] { (byte)i0 };
        CharBuffer out = converter.decode(ByteBuffer.wrap(in));
        if (!(out.length() == 1 && out.charAt(0) == 0xfffd)) {
          System.out.print("0x"+toHexString2(i0)+"\t");
          printOutput(out);
          System.out.println();
        }
      } catch (UnmappableCharacterException e0) {
      } catch (CharacterCodingException e0) {
        if (depth > 1)
          for (int i1 = 0; i1 < 0x100; i1++) {
            try {
              byte[] in = new byte[] { (byte)i0, (byte)i1 };
              CharBuffer out = converter.decode(ByteBuffer.wrap(in));
              if (!(out.length() == 1 && out.charAt(0) == 0xfffd)) {
                System.out.print("0x"+toHexString2(i0)+toHexString2(i1)+"\t");
                printOutput(out);
                System.out.println();
               }
            } catch (UnmappableCharacterException e1) {
            } catch (CharacterCodingException e1) {
              if (depth > 2)
                for (int i2 = 0; i2 < 0x100; i2++) {
                  try {
                    byte[] in = new byte[] { (byte)i0, (byte)i1, (byte)i2 };
                    CharBuffer out = converter.decode(ByteBuffer.wrap(in));
                    if (!(out.length() == 1 && out.charAt(0) == 0xfffd)) {
                      System.out.print("0x"+toHexString2(i0)+toHexString2(i1)+toHexString2(i2)+"\t");
                      printOutput(out);
                      System.out.println();
                    }
                  } catch (UnmappableCharacterException e2) {
                  } catch (CharacterCodingException e2) {
                    if (depth > 3)
                      for (int i3 = 0; i3 < 0x100; i3++) {
                        try {
                          byte[] in = new byte[] { (byte)i0, (byte)i1, (byte)i2, (byte)i3 };
                          CharBuffer out = converter.decode(ByteBuffer.wrap(in));
                          if (!(out.length() == 1 && out.charAt(0) == 0xfffd)) {
                            System.out.print("0x"+toHexString2(i0)+toHexString2(i1)+toHexString2(i2)+toHexString2(i3)+"\t");
                            printOutput(out);
                            System.out.println();
                          }
                        } catch (UnmappableCharacterException e3) {
                        } catch (CharacterCodingException e3) {
                        }
                      }
                  }
                }
            }
          }
      }
    }
  }
}
