#!/usr/local/bin/clisp -C

;; Run this with LD_PRELOAD=/packages/gnu/lib/libiconv_plug.so !

(defun write-8-bit-table (name)
  (let* ((charset (make-encoding :charset name))
         (tmpfile "BIN256"))
    (unwind-protect
      (progn
        (with-open-file (f tmpfile :direction :output
                                   :element-type '(unsigned-byte 8))
          (dotimes (i 256) (write-byte i f))
        )
        (with-open-file (g (concatenate 'string name ".TXT") :direction :output)
          (format g "#~8TName:             ~A to Unicode~%" name)
          (format g "#~8TUnicode version:  3.0~%")
          (format g "#~8TTable version:    1.0~%")
          (format g "#~8TTable format:     Format A~%")
          (format g "#~8TDate:             ~1{~5@*~D-~4@*~2,'0D-~3@*~2,'0D~:}~%"
                    (multiple-value-list (get-decoded-time)))
          (format g "#       Generated by CLISP~%")
          (format g "#~%")
          (with-open-file (f tmpfile :direction :input :external-format charset)
            (dotimes (in 256)
              (let ((out #\uFFFD))
                (file-position f in)
                (ignore-errors (setq out (read-char f)))
                ; CLISP's line-terminator conversion is too intelligent.
                (when (and (eql in 13) (eql out #\u000A)) (setq out #\u000D))
                (unless (eql out #\uFFFD)
                  (format g "0x~2,'0X~C0x~4,'0X~C#~C~A~%"
                            in #\Tab (char-code out) #\Tab
                            #\Tab (sys::unicode-attributes out)
          ) ) ) ) )
          (format t "Created ~A~%" (pathname g))
        )
      )
      (delete-file tmpfile)
) ) )

(mapc #'write-8-bit-table '(
  "ASCII"
  "ISO-8859-1"
  "ISO-8859-2"
  "ISO-8859-3"
  "ISO-8859-4"
  "ISO-8859-5"
  "ISO-8859-6"
  "ISO-8859-7"
  "ISO-8859-8"
  "ISO-8859-9"
  "ISO-8859-10"
  "ISO-8859-13"
  "ISO-8859-14"
  "ISO-8859-15"
  "KOI8-R"
  "KOI8-U"
  "KOI8-RU"
  "CP1250"
  "CP1251"
  "CP1252"
  "CP1253"
  "CP1254"
  "CP1255"
  "CP1256"
  "CP1257"
  "CP850"
  "CP866"
  "MACROMAN"
  "MACCENTRALEUROPE"
  "MACICELAND"
  "MACCROATIAN"
  "MACROMANIA"
  "MACCYRILLIC"
  "MACUKRAINE"
  "MACGREEK"
  "MACTURKISH"
  "MACHEBREW"
  "MACARABIC"
  "MACTHAI"
  "HP-ROMAN8"
  "NEXTSTEP"
  "ARMSCII-8"
  "GEORGIAN-ACADEMY"
  "GEORGIAN-PS"
  "MULELAO-1"
  "CP1133"
  "TIS-620"
  "CP874"
  "VISCII"
  "TCVN"
  "CP1258"
  "JIS_X0201"
))
