/* Copyright (C) 2000-2002, 2004-2005, 2011 Free Software Foundation, Inc.
   This file is part of the GNU LIBICONV Library.

   The GNU LIBICONV Library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   The GNU LIBICONV Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU LIBICONV Library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation, Inc., 51 Franklin Street,
   Fifth Floor, Boston, MA 02110-1301, USA.  */

/* Create a table from Unicode to CHARSET. */

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <winnls.h>
#include <errno.h>

int
main (int argc, char* argv[])
{
  int cd;
  unsigned char newline_buf[10];
  unsigned char defaultchar_buf[10];

  if (argc != 2)
    {
      fprintf (stderr, "Usage: table-to charset\n");
      return 1;
    }
  cd = atoi (argv[1]);

  {
    unsigned short inbuf[1] = { '\n' };
    int result =
      WideCharToMultiByte (cd, 0, inbuf, sizeof (inbuf) / sizeof (inbuf[0]),
                           newline_buf, sizeof (newline_buf), NULL, NULL);
    if (!result)
      {
        fprintf (stderr, "Could not convert newline: error 0x%x\n", (unsigned int) GetLastError ());
        return 1;
      }
    if (result > 1)
      {
        fprintf (stderr, "newline got converted to more than 1 char\n");
        return 1;
      }
  }
  /* Now newline_buf[0] is the representation of a newline.  */

  {
    unsigned short inbuf[2] = { '?', '\n' };
    int result =
      WideCharToMultiByte (cd, 0, inbuf, sizeof (inbuf) / sizeof (inbuf[0]),
                           defaultchar_buf, sizeof (defaultchar_buf), NULL, NULL);
    if (!result)
      {
        fprintf (stderr, "Could not convert question mark: error 0x%x\n", (unsigned int) GetLastError ());
        return 1;
      }
    if (defaultchar_buf[result - 1] != newline_buf[0])
      {
        fprintf (stderr, "While converting question mark: no newline at the end\n");
        return 1;
      }
    if (result - 1 > 1)
      {
        fprintf (stderr, "question mark got converted to more than 1 char\n");
        return 1;
      }
  }
  /* Now defaultchar_buf[0] is the representation of a question mark.  */

  {
    unsigned int uc;
    unsigned char buf[100];
    for (uc = 0; uc < 0x110000; uc++)
      {
        unsigned short inbuf[3];
        size_t inbufcount;
        int result;
        if (uc < 0x10000)
          {
            inbuf[0] = uc;
            inbuf[1] = '\n';
          inbufcount = 2;
          }
        else
          {
            inbuf[0] = 0xd800 + ((uc - 0x10000) >> 10);
            inbuf[1] = 0xdc00 + ((uc - 0x10000) & 0x3ff);
            inbuf[2] = '\n';
            inbufcount = 3;
          }
        result =
          WideCharToMultiByte (cd, 0, inbuf, inbufcount, buf, sizeof (buf), NULL, NULL);
        if (!result)
          {
            if (GetLastError () == ERROR_INVALID_PARAMETER)
              {
                fprintf (stderr, "While converting U+%04X: got ERROR_INVALID_PARAMETER\n", uc);
                return 1;
              }
            if (GetLastError () == ERROR_INVALID_FLAGS)
              {
                fprintf (stderr, "While converting U+%04X: got ERROR_INVALID_FLAGS\n", uc);
                return 1;
              }
            if (GetLastError () == ERROR_INSUFFICIENT_BUFFER)
              {
                fprintf (stderr, "While converting U+%04X: got ERROR_INSUFFICIENT_BUFFER\n", uc);
                return 1;
              }
            if (GetLastError () != ERROR_NO_UNICODE_TRANSLATION)
              {
                fprintf (stderr, "While converting U+%04X: got error 0x%x\n", uc, (unsigned int) GetLastError ());
                return 1;
              }
          }
        else
          {
            if (buf[result - 1] != newline_buf[0])
              {
                fprintf (stderr, "While converting U+%04X: no newline at the end\n", uc);
              }
            else if (!((result - 1 == 1 && buf[0] == defaultchar_buf[0])
                       || (inbufcount == 3 && result - 1 == 2 && buf[0] == defaultchar_buf[0] && buf[1] == defaultchar_buf[0])))
              {
                /* Language tags may silently be dropped.  */
                if (!(result - 1 == 0 && uc >= 0xe0000 && uc < 0xe0080))
                  {
                    unsigned int jmax = result - 1;
                    unsigned int j;
                    printf("0x");
                    for (j = 0; j < jmax; j++)
                      printf("%02X", buf[j]);
                    printf("\t0x%04X\n", uc);
                  }
              }
          }
      }
  }

  if (ferror(stdin) || ferror(stdout) || fclose(stdout))
    {
      fprintf (stderr, "I/O error\n");
      return 1;
    }

  return 0;
}
